from src import email
import pytest

from test.utils import auth as test_auth


@pytest.fixture(scope="session", autouse=True)
def getTestCredentials():
    return test_auth.getTestCredentials()


class TestEmail(test_auth.WithUserAssertingFixture):
    def testGetProfile(self):
        profile = email.getProfile()
        print(profile)
        assert profile['emailAddress'] == 'apatterson189.test@gmail.com'



import pickle
import os.path
from google_auth_oauthlib.flow import InstalledAppFlow
from google.auth.transport.requests import Request
import os
from googleapiclient.errors import HttpError
from pathlib import Path

dirname = os.path.dirname(__file__)
storedTokenLocation = None
storedCredentialsLocation = None


def setStoredCredentialsLocation(location):
    global storedCredentialsLocation
    global storedTokenLocation
    storedCredentialsLocation = location
    storedTokenLocation = None if location == None else _getStoredTokenLocation(
        location)


def clearStoredCredentialsLocation(location):
    setStoredCredentialsLocation(None)


def _getExistingCreds():

    if(storedTokenLocation == None):
        return None

    existingCreds = None
    # The file token.pickle stores the user's access and refresh tokens, and is
    # created automatically when the authorization flow completes for the first
    # time.
    if os.path.exists(storedTokenLocation):
        with open(storedTokenLocation, 'rb') as token:
            existingCreds = pickle.load(token)

    return existingCreds


def _storeCreds(creds):
    if(storedTokenLocation == None):
        raise Exception("stored token locaion not set")

    with open(storedTokenLocation, 'wb') as token:
        pickle.dump(creds, token)


def getCreds(scopes, resetScopes=False):

    if(storedCredentialsLocation == None):
        raise Exception("stored credential locaion not set")

    allScopes = list(scopes)

    creds = _getExistingCreds()

    if creds and not resetScopes:
        allScopes += list(set(creds.scopes))

    # cred exist for all required scope but are expired
    if creds and creds.has_scopes(allScopes):
        if creds.expired and creds.refresh_token:
            creds.refresh(Request())
        # else do nothing, old creds are ok
    else:
        flow = InstalledAppFlow.from_client_secrets_file(
            storedCredentialsLocation, allScopes)
        creds = flow.run_local_server(port=0)
        # Save the credentials for the next run

        _storeCreds(creds)

    return creds


def _getStoredTokenLocation(storedCredentialsLocation):
    return os.path.join(Path(storedCredentialsLocation).parent, 'token.pickle')


def clearStoredToken():
    storedTokenLocation = _getStoredTokenLocation(
        storedCredentialsLocation)
    os.remove(storedTokenLocation)



from src import auth
from src import drive
from src import sheets as spreadsheets
from test.utils import drive as test_drive
from test.utils import auth as test_auth

import pytest


@pytest.fixture(scope="session", autouse=True)
def getTestCredentials():
    return test_auth.getTestCredentials()


class TestDrive(test_drive.WithDriveCleaningFixture):
    def test_create(self):
        testName = 'testing_create_sheet'
        testData = [[]]
        createdSheetId = spreadsheets.create(testData, testName)

        contents = drive.list()
        createdSheet = {'id': createdSheetId, 'name': testName}

        assert contents == [createdSheet]

    def test_get(self):
        testName = 'testing_create_sheet'
        testData = [['apple', 'banana'], ['carrot']]
        createdSheetId = spreadsheets.create(testData, testName)

        spreadsheet = spreadsheets.get(createdSheetId, True)

        assert spreadsheet != None
        assert spreadsheet['spreadsheetId'] == createdSheetId
        assert spreadsheet['properties']['title'] == testName
        sheets = spreadsheet['sheets']

        assert len(sheets) == 1

        gridData = []
        sheetDataRows = sheets[0]['data'][0]['rowData']
        for row in sheetDataRows:
            rowDataValues = row['values']
            rowRawValues = []
            for rowDataValue in rowDataValues:
                stringValue = rowDataValue['userEnteredValue']['stringValue']
                rowRawValues.append(stringValue)
            gridData.append(rowRawValues)

        assert gridData == testData

    def test_batchUpdate(self):
        testName = 'testing_create_sheet'
        testData = [[]]
        createdSheetId = spreadsheets.create(testData, testName)

        testData = [['apple', 'banana'], ['carrot']]

        spreadsheets.batchUpdate(createdSheetId, 'A:C', testData)

        spreadsheet = spreadsheets.get(createdSheetId, True)

        assert spreadsheet != None
        assert spreadsheet['spreadsheetId'] == createdSheetId
        assert spreadsheet['properties']['title'] == testName
        sheets = spreadsheet['sheets']

        assert len(sheets) == 1

        gridData = []
        sheetDataRows = sheets[0]['data'][0]['rowData']
        for row in sheetDataRows:
            rowDataValues = row['values']
            rowRawValues = []
            for rowDataValue in rowDataValues:
                stringValue = rowDataValue['userEnteredValue']['stringValue']
                rowRawValues.append(stringValue)
            gridData.append(rowRawValues)

        assert gridData == testData

    def test_batchUpdate_and_create_new_sheet(self):
        testName = 'testing_create_sheet'
        testData = [[]]
        createdSheetId = spreadsheets.create(testData, testName)

        newSheetName = 'new_sheet'

        spreadsheets.addSheet(createdSheetId, newSheetName)

        spreadsheet = spreadsheets.get(createdSheetId, False)

        assert len(spreadsheet['sheets']) == 2
        assert spreadsheet['sheets'][-1]['properties']['title'] == newSheetName



from . import auth
from googleapiclient.discovery import build


def _getService(scopes):
    creds = auth.getCreds(scopes)
    return build('gmail', 'v1', credentials=creds)


def getReadonlyService():
    return _getService(['https://www.googleapis.com/auth/gmail.readonly'])


def getProfile():
    service = getReadonlyService()
    usersService = service.users()

    request = usersService.getProfile(userId='me')
    profile = request.execute()

    return profile



from src import auth
from src import drive
from test.utils import drive as test_drive
from test.utils import auth as test_auth

import pytest


@pytest.fixture(scope="session", autouse=True)
def getTestCredentials():
    return test_auth.getTestCredentials()


class TestDrive(test_drive.WithDriveCleaningFixture):

    def test_list_empty(self):
        contents = drive.list()
        assert contents == []

    def test_createBlank(self):
        testName = "testing_created_blank"
        createdSheetId = drive.createBlank(testName, [], drive.MimeTypes.sheet)

        contents = drive.list()
        createdSheet = {'id': createdSheetId, 'name': testName}

        assert contents == [createdSheet]

    def test_createBlankSheet(self):
        testName = "testing_created_blank_sheet"
        createdSheetId = drive.createBlankSheet(testName, [])

        contents = drive.list()
        createdSheet = {'id': createdSheetId, 'name': testName}

        assert contents == [createdSheet]

    def test_list_filtered(self):
        testName1 = "testing_created_1"
        createdSheetId1 = drive.createBlank(
            testName1, [], drive.MimeTypes.sheet)

        testName2 = "testing_created_2"
        createdSheetId2 = drive.createBlank(
            testName2, [], drive.MimeTypes.sheet)

        testName3 = "testing_created_3"
        createdSheetId3 = drive.createBlank(
            testName3, [], drive.MimeTypes.sheet)

        matchedContents = drive.list("name = 'testing_created_2'")

        assert matchedContents == [
            {'id': createdSheetId2, 'name': testName2}
        ]



from setuptools import find_packages, setup

setup(
    name='apatterson189_google_api_helpers',
    packages=find_packages(include=['src']),
    version='0.1.0',
    description='LIbrary of helpers for using google apis',
    author='Albert Patterson',
    license='MIT',
    install_requires=[
        'atomicwrites==1.4.0',
        'attrs==20.2.0',
        'autopep8==1.5.4',
        'cachetools==4.1.1',
        'certifi==2020.6.20',
        'chardet==3.0.4',
        'google-api-core==1.22.4',
        'google-api-python-client==1.12.3',
        'google-auth==1.22.1',
        'google-auth-httplib2==0.0.4',
        'google-auth-oauthlib==0.4.1',
        'googleapis-common-protos==1.52.0',
        'httplib2==0.18.1',
        'idna==2.10',
        'importlib-metadata==2.0.0',
        'iniconfig==1.1.1',
        'more-itertools==8.5.0',
        'oauthlib==3.1.0',
        'packaging==20.4',
        'pluggy==0.13.1',
        'protobuf==3.13.0',
        'py==1.9.0',
        'pyasn1==0.4.8',
        'pyasn1-modules==0.2.8',
        'pycodestyle==2.6.0',
        'pyparsing==2.4.7',
        'pytz==2020.1',
        'requests==2.24.0',
        'requests-oauthlib==1.3.0',
        'rsa==4.6',
        'six==1.15.0',
        'toml==0.10.1',
        'uritemplate==3.0.1',
        'urllib3==1.25.10',
        'wcwidth==0.2.5',
        'zipp==3.3.1',
    ],
    setup_requires=['pytest-runner'],
    tests_require=['pytest==6.1.1'],
    test_suite='test',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)



from enum import Enum
from . import auth
from googleapiclient.discovery import build
from googleapiclient.errors import HttpError


class Scopes:
    drive_metadata_readonly = 'https://www.googleapis.com/auth/drive.metadata.readonly'
    drive_metadata = 'https://www.googleapis.com/auth/drive.metadata'
    drive_readonly = 'https://www.googleapis.com/auth/drive.readonly'
    drive = 'https://www.googleapis.com/auth/drive'


class MimeTypes:
    sheet = 'application/vnd.google-apps.spreadsheet'


def _getService(scopes):
    creds = auth.getCreds(scopes)
    return build('drive', 'v3', credentials=creds)


def getMetadataReadonlyService():
    return _getService(['https://www.googleapis.com/auth/drive.metadata.readonly'])


def getMetadataService():
    return _getService(['https://www.googleapis.com/auth/drive.metadata'])


def getReadonlyService():
    return _getService(['https://www.googleapis.com/auth/drive.readonly'])


def getService():
    return _getService(['https://www.googleapis.com/auth/drive'])


def getMetadataReadonlyService():
    return _getService(['https://www.googleapis.com/auth/drive.metadata.readonly'])


def list(q=""):
    service = getMetadataReadonlyService()
    filesService = service.files()

    files = []
    page_token = None
    while True:
        response = filesService.list(q=q,
                                     spaces='drive',
                                     fields='nextPageToken, files(id, name, description)',
                                     pageToken=page_token).execute()
        files += response['files']

        page_token = response.get('nextPageToken', None)
        if page_token is None:
            break
    return files


def createBlank(name, parents, mimeType):
    body = {
        'name': name,
        'parents': parents,
        'mimeType': mimeType,
    }
    response = getService().files().create(body=body).execute()
    return response['id']


def createBlankSheet(name, parents):
    return createBlank(name, parents, 'application/vnd.google-apps.spreadsheet')


def delete(id):
    return getService().files().delete(fileId=id).execute()



from . import auth
from googleapiclient.discovery import build


def _getService(scopes):
    creds = auth.getCreds(scopes)
    return build('sheets', 'v4', credentials=creds)


def getReadonlyService():
    return _getService(['https://www.googleapis.com/auth/spreadsheets.readonly'])


def getService():
    return _getService(['https://www.googleapis.com/auth/spreadsheets'])


def getValues(spreadsheetId, range):
    service = getReadonlyService()
    valuesService = service.spreadsheets().values()
    request = valuesService.get(spreadsheetId=spreadsheetId,
                                range=range)
    response = request.execute()

    values = response.get('values', [])
    return values


def get(spreadsheetId, includeGridData=False):
    service = getReadonlyService()
    spreadsheetsService = service.spreadsheets()
    request = spreadsheetsService.get(
        spreadsheetId=spreadsheetId, includeGridData=includeGridData)
    response = request.execute()

    return response


def batchUpdate(spreadsheetId, range, data):
    service = getService()
    batch_update_values_request_body = {
        'value_input_option': 'RAW',
        'data': [
            {
                'range': range,
                'majorDimension': 'ROWS',
                'values': data
            }
        ],
    }
    valuesService = service.spreadsheets().values()
    request = valuesService.batchUpdate(
        spreadsheetId=spreadsheetId, body=batch_update_values_request_body)

    response = request.execute()
    return response


def addSheet(spreadsheetId, sheetName):

    requests = []

    requests.append({
        "addSheet": {
            "properties": {
                "title": sheetName,
            }
        }
    })

    body = {
        'requests': requests
    }

    service = getService()
    spreadsheetsService = service.spreadsheets()
    request = spreadsheetsService.batchUpdate(
        spreadsheetId=spreadsheetId, body=body)
    response = request.execute()

    return response


def create(data, title='api generated', startRow=0, startColumn=0):
    service = getService()

    rowData = []
    for row in data:
        rowDataCols = []
        for point in row:
            value = {
                "userEnteredValue": {
                    "stringValue": point,
                },
            }
            rowDataCols.append(value)
        rowDataRow = {
            "values": rowDataCols,
        }
        rowData.append(rowDataRow)

    body = {
        "properties": {
            "title": title
        },
        "sheets": [
            {
                "data": {
                    "rowData": rowData,
                    "startRow": startRow,
                    "startColumn": startColumn
                }
            },
        ]
    }
    spreadsheetsService = service.spreadsheets()
    request = spreadsheetsService.create(body=body)
    response = request.execute()
    return response['spreadsheetId']
